@@ -10,16 +10,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack import _log
 from openstack.baremetal.v1 import _common
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
 
 
-_logger = _log.setup_logging('openstack')
-
-
 class ValidationResult(object):
     """Result of a single interface validation.
 
@@ -45,6 +41,7 @@ class Node(_common.ListMixin, resource.Resource):
     allow_commit = True
     allow_delete = True
     allow_list = True
+    allow_patch = True
     commit_method = 'PATCH'
     commit_jsonpatch = True
 
@@ -250,7 +247,7 @@ class Node(_common.ListMixin, resource.Resource):
                 "state %s" % expected_provision_state)
 
         # Ironic cannot set provision_state itself, so marking it as unchanged
-        self._body.clean(only={'provision_state'})
+        self._clean_body_attrs({'provision_state'})
         super(Node, self).create(session, *args, **kwargs)
 
         if (self.provision_state == 'enroll'
@@ -267,6 +264,39 @@ class Node(_common.ListMixin, resource.Resource):
 
         return self
 
+    def commit(self, session, *args, **kwargs):
+        """Commit the state of the instance to the remote resource.
+
+        :param session: The session to use for making this request.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+
+        :return: This :class:`Node` instance.
+        """
+        # These fields have to be set through separate API.
+        if ('maintenance_reason' in self._body.dirty
+                or 'maintenance' in self._body.dirty):
+            if not self.is_maintenance and self.maintenance_reason:
+                if 'maintenance' in self._body.dirty:
+                    self.maintenance_reason = None
+                else:
+                    raise ValueError('Maintenance reason cannot be set when '
+                                     'maintenance is False')
+            if self.is_maintenance:
+                self._do_maintenance_action(
+                    session, 'put', {'reason': self.maintenance_reason})
+            else:
+                # This corresponds to setting maintenance=False and
+                # maintenance_reason=None in the same request.
+                self._do_maintenance_action(session, 'delete')
+
+            self._clean_body_attrs({'maintenance', 'maintenance_reason'})
+            if not self.requires_commit:
+                # Other fields are not updated, re-fetch the node to reflect
+                # the new status.
+                return self.fetch(session)
+
+        return super(Node, self).commit(session, *args, **kwargs)
+
     def set_provision_state(self, session, target, config_drive=None,
                             clean_steps=None, rescue_password=None,
                             wait=False, timeout=None):
@@ -293,16 +323,24 @@ class Node(_common.ListMixin, resource.Resource):
         :return: This :class:`Node` instance.
         :raises: ValueError if ``config_drive``, ``clean_steps`` or
             ``rescue_password`` are provided with an invalid ``target``.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
+            reaches an error state while waiting for the state.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if timeout
+            is reached while waiting for the state.
         """
         session = self._get_session(session)
 
+        version = None
         if target in _common.PROVISIONING_VERSIONS:
             version = '1.%d' % _common.PROVISIONING_VERSIONS[target]
-        else:
-            if config_drive and target == 'rebuild':
+
+        if config_drive:
+            # Some config drive actions require a higher version.
+            if isinstance(config_drive, dict):
+                version = '1.56'
+            elif target == 'rebuild':
                 version = '1.35'
-            else:
-                version = None
+
         version = utils.pick_microversion(session, version)
 
         body = {'target': target}
@@ -366,6 +404,9 @@ class Node(_common.ListMixin, resource.Resource):
             ``manageable`` transition is ``enroll`` again.
 
         :return: This :class:`Node` instance.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
+            reaches an error state and ``abort_on_failed_state`` is ``True``.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` on timeout.
         """
         for count in utils.iterate_timeout(
                 timeout,
@@ -377,10 +418,11 @@ class Node(_common.ListMixin, resource.Resource):
                                          abort_on_failed_state):
                 return self
 
-            _logger.debug('Still waiting for node %(node)s to reach state '
-                          '"%(target)s", the current state is "%(state)s"',
-                          {'node': self.id, 'target': expected_state,
-                           'state': self.provision_state})
+            session.log.debug(
+                'Still waiting for node %(node)s to reach state '
+                '"%(target)s", the current state is "%(state)s"',
+                {'node': self.id, 'target': expected_state,
+                 'state': self.provision_state})
 
     def wait_for_reservation(self, session, timeout=None):
         """Wait for a lock on the node to be released.
@@ -416,9 +458,10 @@ class Node(_common.ListMixin, resource.Resource):
             if self.reservation is None:
                 return self
 
-            _logger.debug('Still waiting for the lock to be released on node '
-                          '%(node)s, currently locked by conductor %(host)s',
-                          {'node': self.id, 'host': self.reservation})
+            session.log.debug(
+                'Still waiting for the lock to be released on node '
+                '%(node)s, currently locked by conductor %(host)s',
+                {'node': self.id, 'host': self.reservation})
 
     def _check_state_reached(self, session, expected_state,
                              abort_on_failed_state=True):
@@ -433,8 +476,8 @@ class Node(_common.ListMixin, resource.Resource):
             ``manageable`` transition is ``enroll`` again.
 
         :return: ``True`` if the target state is reached
-        :raises: SDKException if ``abort_on_failed_state`` is ``True`` and
-            a failure state is reached.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the node
+            reaches an error state and ``abort_on_failed_state`` is ``True``.
         """
         # NOTE(dtantsur): microversion 1.2 changed None to available
         if (self.provision_state == expected_state
@@ -445,7 +488,7 @@ class Node(_common.ListMixin, resource.Resource):
             return False
 
         if self.provision_state.endswith(' failed'):
-            raise exceptions.SDKException(
+            raise exceptions.ResourceFailure(
                 "Node %(node)s reached failure state \"%(state)s\"; "
                 "the last error is %(error)s" %
                 {'node': self.id, 'state': self.provision_state,
@@ -454,7 +497,7 @@ class Node(_common.ListMixin, resource.Resource):
         # "enroll"
         elif (expected_state == 'manageable'
               and self.provision_state == 'enroll' and self.last_error):
-            raise exceptions.SDKException(
+            raise exceptions.ResourceFailure(
                 "Node %(node)s could not reach state manageable: "
                 "failed to verify management credentials; "
                 "the last error is %(error)s" %
@@ -564,8 +607,9 @@ class Node(_common.ListMixin, resource.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
 
         if ignore_missing and response.status_code == 400:
-            _logger.debug('VIF %(vif)s was already removed from node %(node)s',
-                          {'vif': vif_id, 'node': self.id})
+            session.log.debug(
+                'VIF %(vif)s was already removed from node %(node)s',
+                {'vif': vif_id, 'node': self.id})
             return False
 
         msg = ("Failed to detach VIF {vif} from bare metal node {node}"
@@ -643,5 +687,147 @@ class Node(_common.ListMixin, resource.Resource):
         return {key: ValidationResult(value.get('result'), value.get('reason'))
                 for key, value in result.items()}
 
+    def set_maintenance(self, session, reason=None):
+        """Enable maintenance mode on the node.
+
+        :param session: The session to use for making this request.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param reason: Optional reason for maintenance.
+        :return: This :class:`Node` instance.
+        """
+        self._do_maintenance_action(session, 'put', {'reason': reason})
+        return self.fetch(session)
+
+    def unset_maintenance(self, session):
+        """Disable maintenance mode on the node.
+
+        :param session: The session to use for making this request.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :return: This :class:`Node` instance.
+        """
+        self._do_maintenance_action(session, 'delete')
+        return self.fetch(session)
+
+    def _do_maintenance_action(self, session, verb, body=None):
+        session = self._get_session(session)
+        version = self._get_microversion_for(session, 'commit')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'maintenance')
+        response = getattr(session, verb)(
+            request.url, json=body,
+            headers=request.headers, microversion=version)
+        msg = ("Failed to change maintenance mode for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+    def set_boot_device(self, session, boot_device, persistent=False):
+        """Set node boot device
+
+        :param session: The session to use for making this request.
+        :param boot_device: Boot device to assign to the node.
+        :param persistent: If the boot device change is maintained after node
+            reboot
+        :return: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = self._get_microversion_for(session, 'commit')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'management', 'boot_device')
+
+        body = {'boot_device': boot_device, 'persistent': persistent}
+
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to set boot device for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+    def add_trait(self, session, trait):
+        """Add a trait to a node.
+
+        :param session: The session to use for making this request.
+        :param trait: The trait to add to the node.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session, '1.37')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'traits', trait)
+        response = session.put(
+            request.url, json=None,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to add trait {trait} for node {node}"
+               .format(trait=trait, node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        self.traits = list(set(self.traits or ()) | {trait})
+
+    def remove_trait(self, session, trait, ignore_missing=True):
+        """Remove a trait from a node.
+
+        :param session: The session to use for making this request.
+        :param trait: The trait to remove from the node.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the trait does not exist.
+            Otherwise, ``False`` is returned.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session, '1.37')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'traits', trait)
+
+        response = session.delete(
+            request.url, headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        if ignore_missing or response.status_code == 400:
+            session.log.debug(
+                'Trait %(trait)s was already removed from node %(node)s',
+                {'trait': trait, 'node': self.id})
+            return False
+
+        msg = ("Failed to remove trait {trait} from bare metal node {node}"
+               .format(node=self.id, trait=trait))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        self.traits = list(set(self.traits) - {trait})
+
+        return True
+
+    def set_traits(self, session, traits):
+        """Set traits for a node.
+
+        Removes any existing traits and adds the traits passed in to this
+        method.
+
+        :param session: The session to use for making this request.
+        :param traits: list of traits to add to the node.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session, '1.37')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'traits')
+
+        body = {'traits': traits}
+
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to set traits for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        self.traits = traits
+
 
 NodeDetail = Node
