@@ -20,6 +20,8 @@ import datetime
 import munch
 import six
 
+from openstack import resource
+
 _IMAGE_FIELDS = (
     'checksum',
     'container_format',
@@ -38,6 +40,7 @@ _SERVER_FIELDS = (
     'addresses',
     'adminPass',
     'created',
+    'description',
     'key_name',
     'metadata',
     'networks',
@@ -45,6 +48,7 @@ _SERVER_FIELDS = (
     'private_v4',
     'public_v4',
     'public_v6',
+    'server_groups',
     'status',
     'updated',
     'user_id',
@@ -265,11 +269,18 @@ class Normalizer(object):
         return ret
 
     def _normalize_image(self, image):
-        new_image = munch.Munch(
-            location=self._get_current_location(project_id=image.get('owner')))
+        if isinstance(image, resource.Resource):
+            image = image.to_dict(ignore_none=True, original_names=True)
+            location = image.pop(
+                'location',
+                self._get_current_location(project_id=image.get('owner')))
+        else:
+            location = self._get_current_location(
+                project_id=image.get('owner'))
+            # This copy is to keep things from getting epically weird in tests
+            image = image.copy()
 
-        # This copy is to keep things from getting epically weird in tests
-        image = image.copy()
+        new_image = munch.Munch(location=location)
 
         # Discard noise
         self._remove_novaclient_artifacts(image)
@@ -317,7 +328,7 @@ class Normalizer(object):
         new_image['is_protected'] = protected
         new_image['locations'] = image.pop('locations', [])
 
-        metadata = image.pop('metadata', {})
+        metadata = image.pop('metadata', {}) or {}
         for key, val in metadata.items():
             properties.setdefault(key, val)
 
@@ -468,9 +479,11 @@ class Normalizer(object):
 
         # OpenStack can return image as a string when you've booted
         # from volume
-        if str(server['image']) != server['image']:
-            server['image'].pop('links', None)
-        ret['image'] = server.pop('image')
+        image = server.pop('image', None)
+        if str(image) != image:
+            image = munch.Munch(id=image['id'])
+
+        ret['image'] = image
         # From original_names from sdk
         server.pop('imageRef', None)
         # From original_names from sdk
@@ -515,6 +528,13 @@ class Normalizer(object):
                 'OS-EXT-SRV-ATTR:hypervisor_hostname',
                 'OS-EXT-SRV-ATTR:instance_name',
                 'OS-EXT-SRV-ATTR:user_data',
+                'OS-EXT-SRV-ATTR:host',
+                'OS-EXT-SRV-ATTR:hostname',
+                'OS-EXT-SRV-ATTR:kernel_id',
+                'OS-EXT-SRV-ATTR:launch_index',
+                'OS-EXT-SRV-ATTR:ramdisk_id',
+                'OS-EXT-SRV-ATTR:reservation_id',
+                'OS-EXT-SRV-ATTR:root_device_name',
                 'OS-SCH-HNT:scheduler_hints',
         ):
             short_key = key.split(':')[1]
@@ -543,7 +563,10 @@ class Normalizer(object):
             ret['config_drive'] = config_drive
             ret['project_id'] = project_id
             ret['tenant_id'] = project_id
-            ret['region'] = self.config.region_name
+            # TODO(efried): This is hardcoded to 'compute' because this method
+            # should only ever be used by the compute proxy. (That said, it
+            # doesn't appear to be used at all, so can we get rid of it?)
+            ret['region'] = self.config.get_region_name('compute')
             ret['cloud'] = self.config.name
             ret['az'] = az
             for key, val in ret['properties'].items():
@@ -1082,7 +1105,10 @@ class Normalizer(object):
 
     def _normalize_stack(self, stack):
         """Normalize Heat Stack"""
-        stack = stack.copy()
+        if isinstance(stack, resource.Resource):
+            stack = stack.to_dict(ignore_none=True, original_names=True)
+        else:
+            stack = stack.copy()
 
         # Discard noise
         self._remove_novaclient_artifacts(stack)
@@ -1092,7 +1118,10 @@ class Normalizer(object):
         stack.pop('status', None)
         stack.pop('identifier', None)
 
-        stack_status = stack.pop('stack_status')
+        stack_status = None
+
+        stack_status = stack.pop('stack_status', None) or \
+            stack.pop('status', None)
         (action, status) = stack_status.split('_', 1)
 
         ret = munch.Munch(
@@ -1121,13 +1150,13 @@ class Normalizer(object):
                 ('tempate_description', 'template_description'),
                 ('timeout_mins', 'timeout_mins'),
                 ('tags', 'tags')):
-            value = stack.pop(old_name, None)
+            value = stack.get(old_name, None)
             ret[new_name] = value
             if not self.strict_mode:
                 ret[old_name] = value
         ret['identifier'] = '{name}/{id}'.format(
             name=ret['name'], id=ret['id'])
-        ret['properties'] = stack
+        # ret['properties'] = stack
         return ret
 
     def _normalize_machines(self, machines):
@@ -1165,3 +1194,37 @@ class Normalizer(object):
             location=self._get_identity_location(),
             properties={},
         )
+
+    def _normalize_containers(self, containers):
+        """Normalize Swift Containers"""
+        ret = []
+        for container in containers:
+            ret.append(self._normalize_container(container))
+        return ret
+
+    def _normalize_container(self, container):
+        """Normalize Swift Container."""
+
+        return munch.Munch(
+            name=container.get('name'),
+            bytes=container.get('bytes'),
+            count=container.get('count'),
+        )
+
+    def _normalize_objects(self, objects):
+        """Normalize Swift Objects"""
+        ret = []
+        for object in objects:
+            ret.append(self._normalize_object(object))
+        return ret
+
+    def _normalize_object(self, object):
+        """Normalize Swift Object."""
+
+        return munch.Munch(
+            name=object.get('name'),
+            bytes=object.get('_bytes'),
+            content_type=object.get('content_type'),
+            hash=object.get('_hash'),
+            last_modified=object.get('_last_modified'),
+        )
