@@ -112,6 +112,27 @@ is needed:
         compute_api_version='2',
         identity_interface='internal')
 
+From oslo.conf CONF object
+--------------------------
+
+For applications that have an oslo.config ``CONF`` object that has been
+populated with ``keystoneauth1.loading.register_adapter_conf_options`` in
+groups named by the OpenStack service's project name, it is possible to
+construct a Connection with the ``CONF`` object and an authenticated Session.
+
+.. note::
+
+    This is primarily intended for use by OpenStack services to talk amongst
+    themselves.
+
+.. code-block:: python
+
+    from openstack import connection
+
+    conn = connection.Connection(
+        session=session,
+        oslo_config=CONF)
+
 From existing CloudRegion
 -------------------------
 
@@ -249,6 +270,10 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
                  use_direct_get=False,
                  task_manager=None,
                  rate_limit=None,
+                 oslo_conf=None,
+                 service_types=None,
+                 global_request_id=None,
+                 strict_proxies=False,
                  **kwargs):
         """Create a connection to a cloud.
 
@@ -295,18 +320,44 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
             keys as service-type and values as floats expressing the calls
             per second for that service. Defaults to None, which means no
             rate-limiting is performed.
+        :param oslo_conf: An oslo.config CONF object.
+        :type oslo_conf: :class:`~oslo_config.cfg.ConfigOpts`
+            An oslo.config ``CONF`` object that has been populated with
+            ``keystoneauth1.loading.register_adapter_conf_options`` in
+            groups named by the OpenStack service's project name.
+        :param service_types:
+            A list/set of service types this Connection should support. All
+            other service types will be disabled (will error if used).
+            **Currently only supported in conjunction with the ``oslo_conf``
+            kwarg.**
+        :param global_request_id: A Request-id to send with all interactions.
+        :param strict_proxies:
+            If True, check proxies on creation and raise
+            ServiceDiscoveryException if the service is unavailable.
+        :type strict_proxies: bool
+            Throw an ``openstack.exceptions.ServiceDiscoveryException`` if the
+            endpoint for a given service doesn't work. This is useful for
+            OpenStack services using sdk to talk to other OpenStack services
+            where it can be expected that the deployer config is correct and
+            errors should be reported immediately.
+            Default false.
         :param kwargs: If a config is not provided, the rest of the parameters
             provided are assumed to be arguments to be passed to the
             CloudRegion constructor.
         """
         self.config = config
         self._extra_services = {}
+        self._strict_proxies = strict_proxies
         if extra_services:
             for service in extra_services:
                 self._extra_services[service.service_type] = service
 
         if not self.config:
-            if session:
+            if oslo_conf:
+                self.config = cloud_region.from_conf(
+                    oslo_conf, session=session, app_name=app_name,
+                    app_version=app_version, service_types=service_types)
+            elif session:
                 self.config = cloud_region.from_session(
                     session=session,
                     app_name=app_name, app_version=app_version,
@@ -325,6 +376,8 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
 
         self._session = None
         self._proxies = {}
+        self.__pool_executor = None
+        self._global_request_id = global_request_id
         self.use_direct_get = use_direct_get
         self.strict_mode = strict
         # Call the _*CloudMixin constructors while we work on
@@ -345,6 +398,32 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         _orchestration.OrchestrationCloudMixin.__init__(self)
         _security_group.SecurityGroupCloudMixin.__init__(self)
 
+        # Allow vendors to provide hooks. They will normally only receive a
+        # connection object and a responsible to register additional services
+        vendor_hook = kwargs.get('vendor_hook')
+        if not vendor_hook and 'vendor_hook' in self.config.config:
+            # Get the one from profile
+            vendor_hook = self.config.config.get('vendor_hook')
+        if vendor_hook:
+            try:
+                # NOTE(gtema): no class name in the hook, plain module:function
+                # Split string hook into module and function
+                try:
+                    (package_name, function) = vendor_hook.rsplit(':')
+
+                    if package_name and function:
+                        import pkg_resources
+                        ep = pkg_resources.EntryPoint(
+                            'vendor_hook', package_name, attrs=(function,))
+                        hook = ep.resolve()
+                        hook(self)
+                except ValueError:
+                    self.log.warning('Hook should be in the entrypoint '
+                                     'module:attribute format')
+            except (ImportError, TypeError) as e:
+                self.log.warning('Configured hook %s cannot be executed: %s',
+                                 vendor_hook, e)
+
     @property
     def session(self):
         if not self._session:
@@ -389,6 +468,7 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
                 attr_name.replace('-', '_'),
                 property(fget=getter)
             )
+            self.config.enable_service(attr_name)
 
     def authorize(self):
         """Authorize this Connection
@@ -414,6 +494,9 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         if self.__pool_executor:
             self.__pool_executor.shutdown()
 
+    def set_global_request_id(self, global_request_id):
+        self._global_request_id = global_request_id
+
     def __enter__(self):
         return self
 
