@@ -253,3 +253,11 @@ class ValidationException(SDKException):
 
 class TaskManagerStopped(SDKException):
     """Operations were attempted on a stopped TaskManager."""
+
+
+class ServiceDisabledException(ConfigException):
+    """This service is disabled for reasons."""
+
+
+class ServiceDiscoveryException(SDKException):
+    """The service cannot be discovered."""
