@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network import network_service
 from openstack import resource as _resource
 
 
@@ -18,12 +17,11 @@ class AvailabilityZone(_resource.Resource):
     resource_key = 'availability_zone'
     resources_key = 'availability_zones'
     base_path = '/availability_zones'
-    service = network_service.NetworkService()
 
     # capabilities
     allow_create = False
-    allow_get = False
-    allow_update = False
+    allow_fetch = False
+    allow_commit = False
     allow_delete = False
     allow_list = True
 
