@@ -31,13 +31,12 @@ class TestCredential(base.TestCase):
         self.assertEqual('credential', sot.resource_key)
         self.assertEqual('credentials', sot.resources_key)
         self.assertEqual('/credentials', sot.base_path)
-        self.assertEqual('identity', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
-        self.assertEqual('PATCH', sot.update_method)
+        self.assertEqual('PATCH', sot.commit_method)
 
         self.assertDictEqual(
             {
